from django.db.transaction import on_commit

from kinder.webservice.smev3.utils.managers import BaseRequestManager

from concentrator.smev3_v321.models import OrderRequest
from concentrator.smev3_v321.models import UpdateOrderRequest
from concentrator.smev3_v321.models import PendingUpdateOrderRequest

from .request_context import UpdateOrderRequestContext
from .tasks import OrderRequestSendingTask
from .tasks import UpdateOrderRequestSendingTask


class OrderRequestSMEV3RequestManager(BaseRequestManager):
    """Хелпер запуска задачи по отправке запросов OrderRequest СМЭВ 3."""

    task = OrderRequestSendingTask
    model = OrderRequest

    def create_request(self):
        """Создает запрос.

        :return: инстанс созданного запроса
        :rtype: model
        """

        return self.model.objects.get_or_create(**self.get_params())

    def get_params(self):
        """Формирует параметры для создания запроса из контекста.

        :return: словарь данных
        :rtype: dict
        """

        return {
            'declaration_id': self.context.declaration_id,
            'defaults': {'request_order_id': self.context.order_id},
        }

    def apply_async(self):
        """Запускает задачу."""

        request, _ = self.create_request()
        task = self.create_task()

        on_commit(lambda: task.apply_async((request.id, self.context),))


class UpdateOrderRequestSMEV3RequestManager(BaseRequestManager):
    """Хелпер запуска задачи по отправке запросов UpdateOrderRequest СМЭВ 3."""

    task = UpdateOrderRequestSendingTask
    model = UpdateOrderRequest

    def get_params(self):
        """Формирует параметры для создания запроса из контекста.

        :return: словарь данных
        :rtype: dict
        """

        return {
            'declaration_id': self.context.declaration_id,
        }

    def apply_async(self):
        """Запускает задачу."""

        request = self.create_request()
        task = self.create_task()

        on_commit(lambda: task.apply_async((request.id, self.context),))


class DispatchOrderRequestSMEV3RequestManager:
    """Класс для управления данными запроса UpdateOrderRequest.

    Выполняет запуск задачи отправки данных UpdateOrderRequest
    или создание/обновление отложенной задачи по отправке данных.
    """

    def __init__(self, order_request, source_version, data):
        """Конструкор.

        :param order_request: Объект запроса
        :type order_request: OrderRequest
        :param source_version: Версия источника
        :type source_version: int
        :param data: Данные для отправки
        :type data: dict
        """

        self.order_request = order_request
        self.source_version = source_version
        self.data = data

    def run(self):

        if self.order_request.order_id:
            UpdateOrderRequestSMEV3RequestManager(
                UpdateOrderRequestContext(
                    **{**self.data, 'order_id': self.order_request.order_id})
            ).apply_async()
        else:
            PendingUpdateOrderRequest.objects.update_or_create(
                order_request=self.order_request,
                source_version=self.source_version,
                defaults={'data': self.data})
